unit mainu;

// Copyright (c) 1999 by Ziff-Davis, Inc.
// Written by Neil J. Rubenking

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, ActnList, hdObjs, Menus,
  Buttons;

type
  OneBinInfo = record
    HasBin     : Boolean;
    InReg      : Boolean;
    Local      : Boolean;
    BinPercent : Integer;
    BinSize    : Int64;
    BinReq     : Int64;
    DrvSize    : Int64;
  end;
  AllBinInfo = record
    OneSetting : Boolean;
    GlobalInfo : OneBinInfo;
    DrivesInfo : ARRAY[0..25] OF OneBinInfo;
  end;
  THDForm = class(TForm)
    ActionList1       : TActionList;
      aCleanup        : TAction;
      aHelp           : TAction;
      aAbout          : TAction;
      aExit           : TAction;
      aAdd            : TAction;
      aDefaults       : TAction;
      aEdit           : TAction;
      aRem            : TAction;
      aUndo           : TAction;
      aUp             : TAction;
      aDn             : TAction;
      aProps          : TAction;
      aCount          : TAction;
      aCheckAll       : TAction;
      aClearAll       : TAction;
    pmMain            : TPopupMenu;
      mnuEdit         : TMenuItem;
      mnuRem          : TMenuItem;
      mnuUndo         : TMenuItem;
      N2              : TMenuItem;
      mnuProps        : TMenuItem;
      mnuCount        : TMenuItem;
      N3              : TMenuItem;
      mnuUp           : TMenuItem;
      mnuDown         : TMenuItem;
      N4              : TMenuItem;
      mnuCheckAll     : TMenuItem;
      mnuClearAll     : TMenuItem;
      mnuMethodAll    : TMenuItem;
        mnuRecycleAll : TMenuItem;
        mnuBackupAll  : TMenuItem;
        mnuDeleteAll  : TMenuItem;
    pnlButtons        : TPanel;
      btnAdd          : TBitBtn;
      btnDef          : TBitBtn;
      btnEdit         : TBitBtn;
      btnRem          : TBitBtn;
      btnUndo         : TBitBtn;
      btnUp           : TBitBtn;
      btnDown         : TBitBtn;
      btnProps        : TBitBtn;
      btnCount        : TBitBtn;
    pnlmain           : TPanel;
      btnCleanup      : TButton;
      cbTestMode      : TCheckBox;
      btnHelp         : TButton;
      btnAbout        : TButton;
      btnExit         : TButton;
      lvJunk          : TListView;
      Label1          : TLabel;
    StatusBar1        : TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure aCleanupExecute(Sender: TObject);
    procedure aHelpExecute(Sender: TObject);
    procedure aAboutExecute(Sender: TObject);
    procedure aExitExecute(Sender: TObject);
    procedure aAddUpdate(Sender: TObject);
    procedure aAddExecute(Sender: TObject);
    procedure aDefaultsExecute(Sender: TObject);
    procedure aEditExecute(Sender: TObject);
    procedure aRemExecute(Sender: TObject);
    procedure aUndoExecute(Sender: TObject);
    procedure aUpExecute(Sender: TObject);
    procedure aDnExecute(Sender: TObject);
    procedure aPropsExecute(Sender: TObject);
    procedure aCountExecute(Sender: TObject);
    procedure aCheckAllExecute(Sender: TObject);
    procedure pmMainPopup(Sender: TObject);
    procedure mnuMethodAllClick(Sender: TObject);
    procedure mnuSharedAllClick(Sender: TObject);
    procedure lvJunkEdited(Sender: TObject; Item: TListItem;
      var S: String);
    procedure lvJunkChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure lvJunkSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure lvJunkMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnlButtonsMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    DatName       : String;
    DolName       : String;
    LogName       : String;
    TestName      : String;
    UndoObj       : TJunkFileObj;
    VeryBusy      : Integer;
    StopEvent     : THandle;
    CleanFile     : TextFile;
    CleanIdx      : Integer;
    CleanedTypes  : Integer;
    CleanCountOne : Integer;
    CleanSum      : Integer;
    NonPermSum    : Integer;
    CleanErr      : Integer;
    CleanSize     : Int64;
    CleanMethod   : Integer;
    RecFilesBuff  : PChar;
    RecFilesPtr   : PChar;
    abi           : AllBinInfo;
    TestMode      : Boolean;
    TestList      : TStringList;
    BadBinMsg     : String;
    procedure AppOnHint(Sender: TObject);
    procedure SafeAdd(TJ : TJunkFileObj);
    procedure SaveJTToFile(const S : String);
    function  AnyChecked : Boolean;
    procedure ReptCounting(vSizDel : Int64; vNumDir, vNumDel,
      vNumRes, vNumProt : Integer);
    procedure DoneCounting(Sender : TObject);
    procedure DoCleanup(const S : String; fSize : Int64);
    procedure DoCleanTest(const S : String; fSize : Int64);
    procedure DoneCleanup(Sender : TObject);
    procedure DoneCleanTest(Sender : TObject);
    procedure LogOne(LI : TListItem; isTest : Boolean);
  public
    { Public declarations }
    function RecycleBinProblem : Boolean;
  end;

var
  HDForm: THDForm;

implementation

uses Filectrl, ShellAPI, AboutBox, allfuncs, junkwizu,
  Registry, propu, defaultu;

{$R *.DFM}
{$R MORERES.RES}

const
  rbSize = 16384;
  rbMax = rbSize-2;

procedure THDForm.FormCreate(Sender: TObject);
VAR
  TJ : TJunkFileObj;
  TS : TStream;
  hFile : THandle;
begin
  IgnoreSubst := FindCmdLineSwitch('ignoresubst', ['-', '/'], True);//1.01
  TestList := TStringList.Create;
  TestList.Sorted := True;
  // The next line causes ZipMagic to add HDValet to its list
  // of excluded programs 
  hFile := CreateFile('C:\ZMOFFXX', GENERIC_READ, 0, nil,
    OPEN_EXISTING, 0, 0);
  IF hFile <> INVALID_HANDLE_VALUE THEN CloseHandle(hFile);
  Application.onHint := AppOnHint;
  VeryBusy := 0;
  UndoObj  := nil;
  DatName  := ChangeFileExt(Application.ExeName, '.DAT');
  DolName  := ChangeFileExt(Application.ExeName, '.D$$');
  LogName  := ChangeFileExt(Application.ExeName, '.LOG.TXT');
  TestName := ChangeFileExt(Application.ExeName, '.TEST.TXT');
  Application.HelpFile := ChangeFileExt(Application.ExeName, '.HLP');
  GetPosFmEXEName(Application.ExeName, Self, True);
  TS       := nil;
  IF FileExists(DolName) AND (MessageBox(Self.Handle, 'It appears '+
    'that HDValet was not shut down properly. Attempt to restore '+
    'changed data from the previous session?', 'HDValet', MB_YESNO OR
    MB_ICONQUESTION) = idYes) THEN
    TS := TFileStream.Create(DolName, fmOpenread OR
      fmShareExclusive)
  ELSE IF FileExists(DatName) THEN
    begin
      TS := TFileStream.Create(DatName, fmOpenread OR
        fmShareExclusive);
      IF TS.Size = 0 THEN // First time
        begin
          TS.Free;
          TS := nil;
        end;
    end;
  IF Assigned(TS) THEN
    begin
      try
        WHILE TS.Position < TS.Size DO
          begin
            TJ := TJunkFileObj.Create;
            TJ.LoadFromStream(TS);
            WITH lvJunk.Items.Add DO
              begin
                Caption := TJ.typName;
                Checked := TJ.IsChecked;
                Data    := TJ;
              end;
          end;
      finally
        TS.Free;
      end;
    end
  ELSE
    AddAllDefaults(lvJunk, RecycleBinProblem);
end;

procedure THDForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveJTToFile(DatName);
  DeleteFile(DolName);
end;

procedure THDForm.FormDestroy(Sender: TObject);
begin
  SetPosToEXEName(Application.ExeName, Self, True);
end;

procedure THDForm.FormActivate(Sender: TObject);
VAR
  N : Integer;
  AnyDisabled : Integer;
begin
  AnyDisabled := 0;
  Show;
  Application.processMessages;
  IF RecycleBinProblem THEN
    FOR N := 0 TO lvJunk.Items.Count-1 DO
      WITH TJunkFileObj(lvJunk.Items[N].Data) DO
        IF BackType = cuRecycle THEN
          begin
            IF IsChecked THEN Inc(AnyDisabled);
            lvJunk.Items[N].Checked := False;
            IsChecked := False;
          end;
  IF AnyDisabled > 0 THEN
    MessageDlg(Format(BadBinMsg, [Format('%d junk file types that r'+
      'ely on the Recycle Bin have been disabled.', [AnyDisabled])]),
      mtError, [mbOK, mbHelp], 12);
  IF lvJunk.Items.Count > 0 THEN
    lvJunk.Selected := lvJunk.Items[0];
  aAddUpdate(aAdd);
end;

procedure THDForm.aCleanupExecute(Sender: TObject);
VAR
  N : Integer;
  AnyDisabled : Integer;
begin
  IF VeryBusy=2 THEN
    begin
      SetEvent(StopEvent);
      WriteLn(CleanFile, '**** PROCESSING HALTED BY USER ****');
      WriteLn(CleanFile);
      Exit;
    end;
  AnyDisabled := 0;
  IF RecycleBinProblem THEN
    FOR N := 0 TO lvJunk.Items.Count-1 DO
      WITH TJunkFileObj(lvJunk.Items[N].Data) DO
        IF BackType = cuRecycle THEN
          begin
            IF IsChecked THEN
              Inc(AnyDisabled);
            lvJunk.Items[N].Checked := False;
            IsChecked := False;
          end;
  IF AnyDisabled > 0 THEN
    begin
      IF AnyChecked THEN
        begin
        IF MessageDlg(Format(BadBinMsg, [Format('%d junk file types'+
          ' that rely on the Recycle Bin have been disabled.',
          [AnyDisabled])])+#13#10#13#10'Do you want to proceed?',
          mtConfirmation, [mbYes, mbNo, mbHelp], 12) <> idYes THEN
          Exit;
        end
      ELSE
        begin
          MessageDlg(Format(BadBinMsg, [Format('%d junk file types '+
            'that rely on the Recycle Bin have been disabled.'#13+
            #10#13#10'There are no enabled junk file types to pro'+
            'cess.', [AnyDisabled])]), mtError, [mbOK, mbHelp], 12);
          Exit;
        end;
    end;
  TestMode         := cbTestMode.Checked;
  IF TestMode THEN
    FOR N := 0 TO 25 DO
      abi.DrivesInfo[N].BinReq := 0;
  IF TestMode THEN TestList.Clear;
  GetMem(RecFilesBuff, rbSize);
  FillChar(RecFilesBuff^, rbSize, 0);
  RecFilesPtr      := RecFilesBuff;
  lvJunk.ReadOnly  := True;
  Screen.Cursor    := crAppStart;
  VeryBusy         := 2;
  aCleanup.Caption := 'Stop';
  StopEvent := CreateEvent(nil, True, False,
    'PC Magazine HDValet Stop Event');
  IF Testmode THEN
    begin
      AssignFile(CleanFile, TestName);
      Rewrite(CleanFile);
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile, 'HDValet TEST Log - ',
        FormatDateTime('dddddd  hh:mm:ss am/pm', Now));
    end
  ELSE
    begin
      AssignFile(CleanFile, LogName);
      IF FileExists(LogName) THEN Append(CleanFile)
      ELSE Rewrite(CleanFile);
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile, 'HDValet Log - ',
        FormatDateTime('dddddd  hh:mm:ss am/pm', Now));
    end;
  WriteLn(CleanFile, StringOfChar('*', 60));
  CleanIdx     := 0;
  CleanSum     := 0;
  NonPermSum   := 0;
  CleanErr     := 0;
  CleanedTypes := 1;
  CleanSize    := 0;
  // Find the first checked item
  WHILE (CleanIdx < lvJunk.Items.Count) AND
        (NOT lvJunk.Items[CleanIdx].Checked) DO Inc(CleanIdx);
  lvJunk.Selected := lvJunk.Items[CleanIdx];
  // Log it...
  LogOne(lvJunk.Selected, TestMode);
  // ... and do the cleanup
  IF TestMode THEN
    WITH TJunkFileObj(lvJunk.Items[CleanIdx].Data)DO
      ActOnFiles(DoCleanTest, nil, DoneCleanTest, StopEvent,
        ExtractFileDir(Application.ExeName))
  ELSE
    WITH TJunkFileObj(lvJunk.Items[CleanIdx].Data)DO
      ActOnFiles(DoCleanup, nil, DoneCleanup, StopEvent,
        ExtractFileDir(Application.ExeName));
end;

procedure THDForm.aHelpExecute(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure THDForm.aAboutExecute(Sender: TObject);
begin
  WITH TAboutForm.Create(Self) DO
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure THDForm.aExitExecute(Sender: TObject);
begin
  Close;
end;

procedure THDForm.aAddUpdate(Sender: TObject);
begin
  IF VeryBusy > 0 THEN
    begin
      aCleanup.Enabled  := VeryBusy = 2;
      cbTestMode.Enabled := False;
      aAdd.Enabled      := False;
      aDefaults.Enabled := False;
      aRem.Enabled      := False;
      aEdit.Enabled     := False;
      aProps.Enabled    := False;
      aCount.Enabled    := VeryBusy = 1;
      aUndo.Enabled     := False;
      aUp.Enabled       := False;
      aDn.Enabled       := False;
      aCheckAll.Enabled := False;
      aClearAll.Enabled := False;
      mnuMethodAll.Enabled := False;
      aHelp.Enabled     := False;
      aAbout.Enabled    := False;
    end
  ELSE
    begin
      aCleanup.Enabled  := AnyChecked;
      cbTestMode.Enabled := True;
      aAdd.Enabled      := True;
      aDefaults.Enabled := True;
      aRem.Enabled      := lvJunk.Selected <> nil;
      aEdit.Enabled     := lvJunk.Selected <> nil;
      aProps.Enabled    := lvJunk.Selected <> nil;
      aCount.Enabled    := lvJunk.Selected <> nil;
      aUndo.Enabled     := UndoObj <> nil;
      aUp.Enabled       := (lvJunk.Selected <> nil) AND
                           (lvJunk.Selected.Index <> 0);
      aDn.Enabled       := (lvJunk.Selected <> nil) AND
                           (lvJunk.Selected.Index <
                            lvJunk.Items.Count-1);
      aCheckAll.Enabled := lvJunk.Items.Count > 0;
      aClearAll.Enabled := lvJunk.Items.Count > 0;
      mnuMethodAll.Enabled := lvJunk.Items.Count > 0;
      aHelp.Enabled     := True;
      aAbout.Enabled    := True;
    end;
end;

procedure THDForm.aAddExecute(Sender: TObject);
VAR T : TJunkFileObj;
begin
  WITH TTypeWiz.Create(Self) DO
  try
    T := TJunkFileObj.Create;
    SetFromObj(T, RecycleBinProblem);
    T.Free;
    IF ShowModal = idOK THEN
      BEGIN
        T := TJunkFileObj.Create(GetFromObj);
        SafeAdd(T);
        SaveJTToFile(DolName);
      END;
  finally
    free;
  end;
end;

procedure THDForm.aDefaultsExecute(Sender: TObject);
begin
  WITH TDefaultForm.Create(Self) DO
  try
    IF ShowModal = idOK THEN
      begin
        SafeAdd(GetTheObj);
        SaveJTToFile(DolName);
      end;
  finally
    Free;
  end;
end;

procedure THDForm.aEditExecute(Sender: TObject);
VAR T : TJunkFileObj;
begin
  WITH TTypeWiz.Create(Self) DO
  try
    T := TJunkFileObj(lvJunk.Selected.Data);
    SetFromObj(T, RecycleBinProblem);
    IF ShowModal = idOK THEN
      BEGIN
        T.Free;
        T := TJunkFileObj.Create(GetFromObj);
        WITH lvJunk.Selected DO
          begin
            Caption := T.typName;
            Checked := T.IsChecked;
            Data := T;
          end;
        SaveJTToFile(DolName);
      END;
  finally
    free;
  end;
end;

procedure THDForm.aRemExecute(Sender: TObject);
VAR Idx : Integer;
begin
  IF Assigned(UndoObj) THEN UndoObj.Free;
  UndoObj := TJunkFileObj(lvJunk.Selected.Data);
  Idx := lvJunk.Selected.Index;
  lvJunk.Items.Delete(lvJunk.Selected.Index);
  IF lvJunk.Items.Count > 0 THEN
    begin
      IF Idx >= lvJunk.Items.Count THEN
        lvJunk.Selected := lvJunk.Items[Idx-1]
      ELSE lvJunk.Selected := lvJunk.Items[Idx];
    end;
  SaveJTToFile(DolName);
end;

procedure THDForm.aUndoExecute(Sender: TObject);
begin
  IF NOT Assigned(UndoObj) THEN Exit;
  WITH lvJunk.Items.Add DO
    begin
      Caption := UndoObj.typName;
      Data    := TJunkFileObj.Create(UndoObj);
      Checked := UndoObj.IsChecked;
    end;
  UndoObj.Free;
  UndoObj := nil;
  SaveJTToFile(DolName);
end;

procedure THDForm.aUpExecute(Sender: TObject);
VAR
  S       : String;
  TJ, TJs : TJunkFileObj;
  Ck      : Boolean;
begin
  IF lvJunk.Selected = nil THEN Exit;
  TJs := TJunkFileObj(lvJunk.Selected.Data);
  WITH lvJunk.Items[lvJunk.Selected.Index-1] DO
    begin
      S       := Caption;
      TJ      := TJunkFileObj(Data);
      Ck      := Checked;
      Caption := lvJunk.Selected.Caption;
      Checked := lvJunk.Selected.Checked;
      Data    := TJs;
    end;
  lvJunk.Selected.Caption := S;
  lvJunk.Selected.Checked := Ck;
  lvJunk.Selected.Data    := TJ;
  lvJunk.Selected         := lvJunk.Items[lvJunk.Selected.Index-1];
  lvJunk.ItemFocused      := lvJunk.Selected;
end;

procedure THDForm.aDnExecute(Sender: TObject);
VAR
  S       : String;
  TJ, TJs : TJunkFileObj;
  Ck      : Boolean;
begin
  IF lvJunk.Selected = nil THEN Exit;
  TJs := TJunkFileObj(lvJunk.Selected.Data);
  WITH lvJunk.Items[lvJunk.Selected.Index+1] DO
    begin
      S       := Caption;
      TJ      := TJunkFileObj(Data);
      Ck      := Checked;
      Caption := lvJunk.Selected.Caption;
      Checked := lvJunk.Selected.Checked;
      Data    := TJs;
    end;
  lvJunk.Selected.Caption := S;
  lvJunk.Selected.Checked := Ck;
  lvJunk.Selected.Data    := TJ;
  lvJunk.Selected         := lvJunk.Items[lvJunk.Selected.Index+1];
  lvJunk.ItemFocused      := lvJunk.Selected;
end;

procedure THDForm.aPropsExecute(Sender: TObject);
begin
  WITH TPropForm.Create(Self) DO
  try
    SetFromObj(TJunkFileObj(lvJunk.Selected.Data));
    ShowModal;
  finally
    Free;
  end;
end;

procedure THDForm.aCountExecute(Sender: TObject);
VAR TJ : TJunkFileObj;
begin
  IF VeryBusy=1 THEN
    begin
      SetEvent(StopEvent);
      Exit;
    end;
  IF lvJunk.Selected = nil THEN Exit;
  IF lvJunk.Selected.Data = nil THEN Exit;
  TJ := TJunkFileObj(lvJunk.Selected.Data);
  CleanIdx        := lvJunk.Selected.Index;
  lvJunk.ReadOnly := True;
  Screen.Cursor   := crAppStart;
  VeryBusy        := 1;
  aCount.Caption  := 'Stop';
  aCount.Hint     := 'Stop counting files';
  btnCount.Font.Style := [fsBold];
  StopEvent := CreateEvent(nil, True, False,
    'PC Magazine HDValet Stop Event');
  CleanSum := 0;
  NonPermSum := 0;
  TJ.ActOnFiles(nil, ReptCounting, DoneCounting, StopEvent,
    ExtractFileDir(Application.ExeName));
end;

procedure THDForm.aCheckAllExecute(Sender: TObject);
VAR
  N : Integer;
  YesCheck : Boolean;
begin
  yesCheck := (Sender AS TAction).Tag = 1;
  FOR N := 0 TO lvJunk.Items.Count-1 DO
    WITH lvJunk.Items[N], TJunkFileObj(Data) DO
      begin
        Checked := YesCheck;
        IsChecked := YesCheck;
      end;
end;

procedure THDForm.pmMainPopup(Sender: TObject);
begin
  aAddUpdate(aAdd);
end;

procedure THDForm.mnuMethodAllClick(Sender: TObject);
begin
  mnuRecycleAll.Enabled := NOT RecycleBinProblem;
end;

procedure THDForm.mnuSharedAllClick(Sender: TObject);
VAR N : Integer;
begin
  FOR N := 0 TO lvJunk.Items.Count-1 DO
    WITH TJunkFileObj(lvJunk.Items[N].Data) DO
      BackType := (Sender AS TMenuItem).Tag;
end;

procedure THDForm.lvJunkEdited(Sender: TObject; Item: TListItem;
  var S: String);
begin
  IF S = '' THEN S := Item.Caption;
  WITH TJunkFileObj(Item.Data) DO
    IF S <> typName THEN
      begin
        typName := S;
        SaveJTToFile(DolName);
      end;
end;

procedure THDForm.lvJunkChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  IF (Change <> ctState) OR (Item.Data = nil) THEN Exit;
  IF lvJunk.ReadOnly THEN Exit;
  IF VeryBusy > 0 THEN
    Item.Checked := TJunkFileObj(Item.Data).IsChecked
  ELSE
    begin
      IF (TJunkFileObj(Item.Data).BackType = cuRecycle) AND
        Item.Checked AND RecycleBinProblem THEN
        begin
          Item.Checked := False;
          MessageDlg(Format(BadBinMsg, ['This junk file type sends '+
            'files to the Recycle Bin. You cannot enable it at this'+
            ' time.']), mtError, [mbOK, mbHelp], 12);
        end
      ELSE TJunkFileObj(Item.Data).IsChecked := Item.Checked;
    end;
end;

procedure THDForm.lvJunkSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  IF (VeryBusy > 0) AND Selected THEN
    begin
      IF Item.Index <> CleanIdx THEN
        begin
          lvJunk.Selected := lvJunk.Items[CleanIdx];
          lvJunk.ItemFocused := lvJunk.Selected;
        end;
    end
  ELSE aAddUpdate(aAdd);
end;

procedure THDForm.lvJunkMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
VAR LI : TListItem;
begin
  IF htOnStateIcon IN lvJunk.GetHitTestInfoAt(X, Y) THEN
    begin
      LI := lvJunk.GetItemAt(X,Y);
      IF LI <> nil THEN
        begin
          LI.Selected := True;
          LI.Focused  := True;
        end;
    end;
end;

procedure THDForm.pnlButtonsMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
// This enables flyover hints for disabled controls
VAR
  P : TPoint;
  N : Integer;
begin
  P := Point(X,Y);
  WITH Sender AS TWinControl DO
    begin
      Hint := '';
      FOR N := 0 TO ControlCount-1 DO
        IF Controls[N] IS TWinControl THEN
          IF PtInRect(TWinControl(Controls[N]).BoundsRect, P) THEN
            Hint := TWinControl(Controls[N]).Hint + ' (disabled)';
      IF Hint = '' THEN
        Application.CancelHint;
    end;
end;



procedure THDForm.AppOnHint(Sender: TObject);
begin
  IF VeryBusy > 1 THEN Exit;

  IF Screen.ActiveForm IS THDForm THEN
    Self.StatusBar1.Panels[1].Text := Application.Hint
  ELSE IF Screen.ActiveForm IS TTypeWiz THEN
    TTypeWiz(Screen.ActiveForm).StatusBar1.SimpleText :=
      Application.Hint

end;

procedure THDForm.SafeAdd(TJ: TJunkFileObj);
VAR N : Integer;
begin
  FOR N := 0 TO lvJunk.Items.Count-1 DO
    IF TJunkFileObj(lvJunk.Items[N].Data).SameAs(TJ) THEN
      begin
        IF MessageBox(Self.Handle, PChar(Format('You are trying '+
          'to add a junk file type that is identical in function '+
          'to the existing junk file type "%s". Continue?',
          [lvJunk.Items[N].Caption])), 'HDValet', MB_YESNO OR
          MB_ICONQUESTION) <> idYes THEN Exit;
      end;
  WITH lvJunk.Items.Add DO
    begin
      Data    := TJ;
      Caption := TJ.TypName;
      Checked := TJ.IsChecked;
    end;
  // Without the next three lines, the listview gets an
  // unwanted horizontal scroll bar at the time a vertical
  // scrollbar is first needed. It's the AutoSize property
  // of the single column that does it. Resizing the window
  // fixes the problem, so we just resize right here.
  N := Self.Width;
  Self.Width := N+1;
  Self.Width := N;  
end;

procedure THDForm.SaveJTToFile(const S : String);
VAR
  N  : Integer;
  TS : TStream;
begin
  TS := TFileStream.Create(S, fmCreate);
  try
    FOR N := 0 TO lvJunk.Items.Count-1 DO
      WITH lvJunk.Items[N] DO
        IF Data <> nil THEN
          WITH TJunkFileObj(Data) DO
            SaveToStream(TS);
  finally
    TS.Free;
  end;
end;

procedure THDForm.DoneCounting(Sender: TObject);
begin
  WITH Sender AS TJunkDelThread DO
    IF WasStopped THEN
      ShowMessage('Operation stopped by user');
  ResetEvent(StopEvent);
  CloseHandle(StopEvent);    
  aCount.Caption := 'Coun&t';
  aCount.Hint    := 'Count the number of files matching the '+
    'selected junk file type';
  btnCount.Font.Style := [];
  Screen.Cursor   := crDefault;
  lvJunk.ReadOnly := False;
  VeryBusy        := 0;
  lvJunk.Selected := lvJunk.Items[CleanIdx];
  CleanIdx        := -1;
end;

procedure THDForm.ReptCounting(vSizDel : Int64; vNumDir, vNumDel,
  vNumRes, vNumProt : Integer);
const
  t1 : ARRAY[Boolean] OF String = ('s were', ' was');
  t2 : ARRAY[Boolean] OF String = ('s', '');
  t3 : ARRAY[Boolean] OF String = ('These files', 'This file');
  t4 : ARRAY[Boolean] OF String = ('do', 'does');
  t5 : ARRAY[Boolean] OF String = ('occupy', 'occupies');
VAR
  M         : String;
  ActualDel : Integer;
begin
  M := Format('HDValet has searched for files matching the '+
    'selected type'#13#10#13#10'%d folder%s searched'#13#10,
    [vNumDir, t1[vNumDir=1]]);
  IF vNumDel = 0 THEN
    M := M + 'No files matched the junk file specifications'
  ELSE
    begin
      M := M + Format('%d file%s matched the junk file '+
        'specifications'#13#10, [vNumDel, t2[vNumDel=1]]);
      M := M + Format('%d file%s did not meet the date or size '+
        'restrictions'#13#10, [vNumRes, t2[vNumRes=1]]);
      M := M + Format('%d file%s protected by the protect file '+
        'specifications'#13#10#13#10, [vNumProt, t1[vNumProt=1]]);
      ActualDel := vNumDel - vNumRes - vNumProt;
      IF ActualDel < 0 THEN ActualDel := 0;
      M := M + Format('%d file%s would be processed'#13#10,
        [ActualDel, t2[ActualDel=1]]);
      IF ActualDel > 0 THEN
        begin
          IF vSizDel = 0 THEN
            M := M + Format('%s %s not occupy any disk space',
              [t3[ActualDel=1], t4[ActualDel=1]])
          ELSE
            M := M + Format('%s %s %s of disk space.',
              [t3[ActualDel=1], t5[ActualDel=1],
              StringForBytes(vSizDel)]);
        end;
    end;
  MessageBox(Self.Handle, PChar(M), 'HDValet',
    MB_OK OR MB_ICONINFORMATION);
end;

procedure THDForm.DoCleanup(const S: String; fSize : Int64);

  procedure DoRecycle;
  VAR
    FOS          : TSHFileOpStruct;
    RecFilesTemp : PChar;
    Rslt         : Integer;

    function FSiz(const fname : String) : Int64;
    VAR
      FD : TWin32FindData;
      FH : THandle;
    begin
      FH := FindFirstFile(PChar(fName), FD);
      IF FH = INVALID_HANDLE_VALUE THEN Result := 0
      ELSE
        try
          Result := FD.nFileSizeHigh;
          Result := Result SHL 32;
          Result := Result + FD.nFileSizeLow;
        finally
          CloseHandle(FH);
        end;
    end;
  begin
    // If S is empty, or the buffer is full, process the files.
    // If any one of them isn't deleted, all of them later in the
    // buffer were skipped, so try again starting after the
    // bad one
    IF (S = '') OR
       (Length(S) + (RecFilesPtr - RecFilesBuff) > rbMax) THEN
      begin
        RecFilesTemp := RecFilesBuff;
        REPEAT
          FillChar(FOS, SizeOf(FOS), 0);
          WITH FOS DO
            begin
              Wnd    := Handle;
              wFunc  := FO_DELETE;
              pFrom  := RecFilesTemp;
              fFlags := FOF_ALLOWUNDO OR FOF_NOCONFIRMATION OR
                FOF_NOERRORUI;
            end;
          Rslt := ShFileOperation(FOS);
          // Report files that are now correctly absent. If one of
          // them wasn't deleted, report THAT, and try again to
          // process all files after it.
          RecFilesPtr := RecFilesTemp;
          WHILE RecFilesPtr[0] <> #0 DO
            begin
              IF FileExists(StrPas(RecFilesPtr)) THEN
                begin
                  WriteLn(Cleanfile, 'ERR', #9, StrPas(RecFilesPtr));
                  WriteLn(CleanFile, #9'* ', SysErrorMessage(Rslt));
                  Dec(CleanSum);
                  Dec(NonPermSum);
                  Dec(CleanCountOne);
                  Inc(CleanErr);
                  CleanSize := CleanSize - FSiz(StrPas(RecFilesPtr));
                  RecFilesPtr := StrEnd(RecFilesPtr) + 1;
                  // Skip the bad one, try again
                  RecFilesTemp := RecFilesPtr;
                  Break;
                end
              ELSE WriteLn(CleanFile, #9, StrPas(RecFilesPtr));
              RecFilesPtr := StrEnd(RecFilesPtr) + 1;
            end;
        UNTIL RecFilesPtr[0] = #0;
        FillChar(RecFilesBuff^, rbSize, 0);
        RecFilesPtr := RecFilesBuff;
      end;
    IF (S <> '') AND FileExists(S) THEN
      begin
        StrCopy(RecFilesPtr, PChar(S));
        RecFilesPtr := StrEnd(RecFilesPtr)+1;
        Inc(CleanSum);
        Inc(NonPermSum);
        Inc(CleanCountOne);
        StatusBar1.Panels[0].Text := IntToStr(CleanCountOne);
        CleanSize := CleanSize + fSize;
      end;
  end;

  procedure DoHolding;
  VAR NewName : String;
  begin
    NewName := S;
    Insert('\HDVal$$$.$$$', NewName, 3);
    ForceDirectories(ExtractFileDir(NewName));
    IF FileExists(NewName) THEN
      begin
        SetFileAttributes(PChar(NewName), 0);
        DeleteFile(NewName);
      end;
    IF RenameFile(S, NewName) THEN
      begin
        WriteLn(CleanFile, #9, S);
        Inc(CleanSum);
        Inc(NonPermSum);
        Inc(CleanCountOne);
        StatusBar1.Panels[0].Text := IntToStr(CleanCountOne);
        CleanSize := CleanSize + fSize;
      end
    ELSE
      begin
        WriteLn(Cleanfile, 'ERR', #9, S);
        WriteLn(CleanFile, #9'* ', SysErrorMessage(GetLastError));
        Inc(CleanErr);
      end;
  end;

  procedure DoDelete;
  begin
    IF DeleteFile(S) THEN
      begin
        WriteLn(CleanFile, #9, S);
        Inc(CleanSum);
        Inc(CleanCountOne);
        StatusBar1.Panels[0].Text := IntToStr(CleanCountOne);
        CleanSize := CleanSize + fSize;
      end
    ELSE
      begin
        WriteLn(Cleanfile, 'ERR', #9, S);
        WriteLn(CleanFile, #9'* ', SysErrorMessage(GetLastError));
        Inc(CleanErr);
      end;
  end;

begin
  CASE CleanMethod OF
    cuRecycle : DoRecycle;
    cuHolding : DoHolding;
    cuDelete  : DoDelete;
  END;
end;

procedure THDForm.DoCleanTest(const S: String; fSize : Int64);
begin
  IF S = '' THEN Exit;
  IF TestList.IndexOf(S) = -1 THEN
    begin
      TestList.Add(S);
      WriteLn(CleanFile, #9, S);
      Inc(CleanSum);
      IF CleanMethod <> cuDelete THEN
        Inc(NonPermSum);
      Inc(CleanCountOne);
      StatusBar1.Panels[0].Text := IntToStr(CleanCountOne);
      CleanSize := CleanSize + fSize;
      IF CleanMethod = cuRecycle THEN
        WITH abi.DrivesInfo[Ord(UpCase(S[1]))-Ord('A')] DO
          binReq := binReq + fSize;
    end
  ELSE WriteLn(CleanFile, #9, '(', S, ') dupe');
end;

procedure THDForm.DoneCleanup(Sender: TObject);
VAR M : String;
begin
  IF CleanMethod = cuRecycle THEN
    DoCleanup('', 0);
  Inc(CleanIdx);
  WHILE (CleanIdx < lvJunk.Items.Count) AND
        (NOT lvJunk.Items[CleanIdx].Checked) DO Inc(CleanIdx);
  IF (NOT (Sender AS TJunkDelThread).WasStopped) AND
     (CleanIdx < lvJunk.Items.Count) THEN // do another
    begin
      Inc(CleanedTypes);
      ResetEvent(StopEvent);
      lvJunk.Selected := lvJunk.Items[CleanIdx];
      WriteLn(CleanFile);
      LogOne(lvJunk.Selected, False);
      WITH TJunkFileObj(lvJunk.Items[CleanIdx].Data)DO
        ActOnFiles(DoCleanup, nil, DoneCleanup, StopEvent,
          ExtractFileDir(Application.ExeName));
    end
  ELSE // done
    begin
      ResetEvent(StopEvent);
      CloseHandle(StopEvent);
      FreeMem(RecFilesBuff);
      RecFilesBuff := nil;
      RecFilesPtr  := nil;
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile, 'HDValet finished at ',
        FormatDateTime('hh:mm:ss am/pm', Now));
      WriteLn(CleanFile, Format('Processed %d junk file types and '+
        'removed %d junk files.', [CleanedTypes, CleanSum]));
      IF CleanSize > 0 THEN
        WriteLn(CleanFile, Format('TOTAL: %s  (files may occupy '+
          'more space on disk)', [StringForBytes(CleanSize)]));
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile);
      WriteLn(CleanFile);
      CloseFile(CleanFile);
      StatusBar1.Panels[0].Text := '';
      StatusBar1.Panels[1].Text := '';
      VeryBusy := 0;
      lvJunk.ReadOnly  := False;
      Screen.Cursor    := crDefault;
      aCleanup.Caption := '&Clean up';
      IF (Sender AS TJunkDelThread).WasStopped THEN
        M := 'Before you stopped it, '
      ELSE M := '';
      M := M + Format('HDValet processed %d junk file types and '+
        'removed %d junk files.', [CleanedTypes, CleanSum]);
      IF CleanErr > 0 THEN
        M := M + Format(' %d files could not be removed. Check '+
          'the log for details.', [CleanErr]);
      IF CleanSize > 0 THEN
        begin
          IF NonPermSum > 0 THEN
            M := M + Format(#13#10'When all of these files are '+
            'permanently deleted, you will realize a savings of at '+
            'least %s', [StringForBytes(CleanSize)])
          ELSE
            M := M + Format(#13#10'By permanently deleting these '+
            'files you have realized a savings of at '+
            'least %s', [StringForBytes(CleanSize)]);
        end;
      M := M + #13#10#13#10'Do you want to view the log file?';
      IF MessageDlg(M, mtConfirmation, [mbYes, mbNo, mbHelp],
        13) = idYes THEN
          ShellExecute(Self.Handle, nil, PChar(LogName), nil,
            nil, SW_SHOWNORMAL);
    end;
end;

procedure THDForm.DoneCleanTest(Sender: TObject);
const
  mdTyp : ARRAY[Boolean] OF TMsgDlgType = (mtConfirmation, mtError);
  mdCtx : ARRAY[Boolean] OF Integer = (14, 12);
  tf : ARRAY[Boolean] OF String = ('False', 'True');
VAR
  N : Integer;
  M : String;
  BinProb : Boolean;
begin
  IF CleanMethod = cuRecycle THEN
    DoCleanTest('', 0);
  Inc(CleanIdx);
  WHILE (CleanIdx < lvJunk.Items.Count) AND
        (NOT lvJunk.Items[CleanIdx].Checked) DO Inc(CleanIdx);
  IF (NOT (Sender AS TJunkDelThread).WasStopped) AND
     (CleanIdx < lvJunk.Items.Count) THEN // do another
    begin
      Inc(CleanedTypes);
      ResetEvent(StopEvent);
      lvJunk.Selected := lvJunk.Items[CleanIdx];
      WriteLn(CleanFile);
      LogOne(lvJunk.Selected, True);
      WITH TJunkFileObj(lvJunk.Items[CleanIdx].Data)DO
        ActOnFiles(DoCleanTest, nil, DoneCleanTest, StopEvent,
          ExtractFileDir(Application.ExeName));
    end
  ELSE // done
    begin
      ResetEvent(StopEvent);
      CloseHandle(StopEvent);
      FreeMem(RecFilesBuff);
      RecFilesBuff := nil;
      RecFilesPtr  := nil;
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile, 'HDValet finished testing at ',
        FormatDateTime('hh:mm:ss am/pm', Now));
      WriteLn(CleanFile, Format('Processed %d junk file types and '+
        'found %d junk files.', [CleanedTypes, CleanSum]));
      IF CleanSize > 0 THEN
        WriteLn(CleanFile, Format('TOTAL: %s  (files may occupy '+
          'more space on disk)', [StringForBytes(CleanSize)]));
      WriteLn(CleanFile, StringOfChar('*', 60));
      WriteLn(CleanFile, 'Recycle Bin statistics:');
      BinProb := False;
      FOR N := 0 TO 25 DO
        WITH abi.DrivesInfo[N] DO
          IF InReg AND Local THEN
            begin
              IF BinReq > BinSize THEN
                begin
                  IF drvSize = 0 THEN
                    WriteLn(CleanFile, Format('%s:\ %12s, '+
                      'Size: %12s (%.02d%%) *ERROR* (??%%)',
                      [Char(N+Ord('a')), StringForBytes(BinReq),
                      StringForBytes(BinSize), BinPercent]))
                  ELSE
                    WriteLn(CleanFile, Format('%s:\ %12s, '+
                      'Size: %12s (%.02d%%) *ERROR* (%.02d%%)',
                      [Char(N+Ord('a')), StringForBytes(BinReq),
                      StringForBytes(BinSize), BinPercent,
                      ((BinReq*100) DIV DrvSize)+1]));
                  binProb := True;
                end
              ELSE
                WriteLn(CleanFile, Format('%s:\ %12s, '+
                  'Size: %12s (%.02d%%)', [Char(N+Ord('a')),
                  StringForBytes(BinReq), StringForBytes(BinSize),
                  BinPercent]));
            end;
      WriteLn(CleanFile, StringOfChar('*', 60));
      CloseFile(CleanFile);
      StatusBar1.Panels[0].Text := '';
      StatusBar1.Panels[1].Text := '';
      VeryBusy := 0;
      lvJunk.ReadOnly  := False;
      Screen.Cursor    := crDefault;
      aCleanup.Caption := '&Clean up';
      IF (Sender AS TJunkDelThread).WasStopped THEN
        M := 'Before you stopped it, '
      ELSE M := '';
      M := M + Format('HDValet tested %d junk file types and '+
        'found %d junk files.', [CleanedTypes, CleanSum]);
      IF CleanSize > 0 THEN
        M := M + Format(#13#10'If all of these files were '+
        'permanently deleted, you would realize a savings of at '+
        'least %s.', [StringForBytes(CleanSize)]);
      IF binProb THEN
        M := M + #13#10#13#10'WARNING: The Recycle Bin is set '+
          'too small on at least one of your drives. If you '+
          'were to clean up this set of junk file types, you '+
          'would not be able to restore all of the files that '+
          'were sent to the Recycle Bin.'#13#10#13#10'You will '+
          'find Recycle Bin statistics at the end of the test '+
          'log.';
      M := M + #13#10#13#10'Do you want to view the test log?';
      IF MessageDlg(M, mdTyp[binProb], [mbYes, mbNo, mbHelp],
        mdCtx[binProb]) = idYes THEN
        ShellExecute(Self.Handle, nil, PChar(TestName), nil,
          nil, SW_SHOWNORMAL);
    end;
end;

function THDForm.AnyChecked : Boolean;
VAR N : Integer;
begin
  Result := False;
  FOR N := 0 TO lvJunk.Items.Count-1 DO
    IF lvJunk.Items[N].Checked THEN
      begin
        Result := True;
        Exit;
      end;
end;

procedure THDForm.LogOne(LI: TListItem; isTest : Boolean);
begin
  IF isTest THEN
    StatusBar1.Panels[1].Text := 'Now testing '+LI.Caption +'...'
  ELSE
    StatusBar1.Panels[1].Text := 'Now processing '+LI.Caption +'...';
  CleanCountOne := 0;
  StatusBar1.Panels[0].text := IntToStr(CleanCountOne);
  IF isTest THEN
    Write(CleanFile, '"',LI.Caption, '" files would have been ')
  ELSE
    Write(CleanFile, '"',LI.Caption, '" files have been ');
  CleanMethod := TJunkFileObj(LI.Data).BackType;
  CASE CleanMethod OF
    cuRecycle : WriteLn(Cleanfile, 'sent to Recycle Bin');
    cuHolding : WriteLn(Cleanfile, 'moved to holding folder');
    cuDelete  : WriteLn(Cleanfile, 'deleted permanently');
  END;
  WriteLn(CleanFile, StringOfChar('-', 60));
end;

function THDForm.RecycleBinProblem : Boolean;
type
  TPurgeBinary = ARRAY[0..$47] OF Byte;
  TPurgeInfo = packed record
    cbSize         : Integer;
    UseOne         : Integer;
    DrivePercents  : ARRAY[0..25] OF Word;
    DefaultPercent : Word;
    GlobalPercent  : Word;
    NoBinDrives    : DWord;
    Signature      : DWord;
  end;
const
  DefPurge : TPurgeInfo = (cbSize: SizeOf(TPurgeInfo); UseOne: 1;
    DrivePercents: (10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
    10,10,10,10,10,10,10,10,10,10); DefaultPercent: 10;
    GlobalPercent: 10; NoBinDrives: 0; Signature: 0);
  WasPurge : TPurgeBinary = (72,0,0,0,1,0,0,0,10,0,10,0,10,0,10,0,10,
    0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,
    10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,10,0,0,0,0,0,
    0,0,0,0);
const
  Problem : Integer = -1;
VAR
  N, L : Integer;
  C : Char;
  S : String;
  PurInfo : TPurgeInfo;
  IsPurge : TPurgeBinary;
  Changed : Boolean;

  procedure BinInfoFromPurgeInfo;
  VAR
    N          : Integer;
    vFreeAvailable,
    vTotalSpace,
    vTotalFree : Int64;
    root       : ARRAY[0..3] OF Char;
    NBD        : DWord;
  begin
    abi.OneSetting := PurInfo.UseOne = 1;
    abi.GlobalInfo.HasBin := PurInfo.NoBinDrives AND (1 SHL 27) = 0;
    abi.GlobalInfo.BinPercent := PurInfo.GlobalPercent;
    IF abi.OneSetting THEN
      begin
        IF abi.GlobalInfo.HasBin THEN
          FOR N := 0 TO 25 DO
            WITH abi.DrivesInfo[N] DO
              begin
                HasBin := True;
                BinPercent := abi.GlobalInfo.BinPercent;
              end;
      end
    ELSE
      begin
        NBD := PurInfo.NoBinDrives;
        FOR N := 0 TO 25 DO
          WITH abi.DrivesInfo[N] DO
            begin
              HasBin := NOT Odd(NBD);
              BinPercent := PurInfo.DrivePercents[N];
              NBD := NBD DIV 2;
            end;
      end;
    strCopy(root, 'a:\');
    FOR N := 0 TO 25 DO
      WITH abi.DrivesInfo[N] DO
        begin
          IF HasBin AND InReg AND (DrvSize = 0) THEN
            begin
              IF GetDiskFreeSpaceEx(root,
                vFreeAvailable, vTotalSpace, @vTotalFree) THEN
                DrvSize := vTotalSpace
              ELSE DrvSize := -1;
            end;
          IF HasBin AND InReg AND (DrvSize > 0) THEN
            BinSize := (DrvSize * BinPercent) DIV 100;
          inc(root[0]);
        end;
  end;

begin
  Move(DefPurge, PurInfo, SizeOf(PurInfo));
  Move(DefPurge, IsPurge, SizeOf(IsPurge));
  WITH TRegistry.Create DO
  try
    RootKey := HKEY_LOCAL_MACHINE;
    IF OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\explorer\'+
      'BitBucket', False) THEN
      begin
        ReadBinaryData('PurgeInfo', IsPurge, SizeOf(TPurgeBinary));
        ReadBinaryData('PurgeInfo', PurInfo, SizeOf(TPurgeInfo));
      end;
    Changed := (Problem = -1) OR (NOT CompareMem(@WasPurge, @IsPurge,
      SizeOf(TPurgeBinary)));
    IF Changed THEN
      begin
        Move(IsPurge, WasPurge, SizeOf(WasPurge));
        FOR C := 'A' to 'Z' DO
          begin
            IF ValueExists(C) THEN
              abi.DrivesInfo[Ord(C)-Ord('A')].InReg := True;
            IF GetDriveType(PChar(C+':\')) = DRIVE_FIXED THEN
              abi.DrivesInfo[Ord(C)-Ord('A')].Local := True;
          end;
      end;
  finally
    Free;
  end;
  IF Changed THEN
    begin
      BinInfoFromPurgeInfo;
      BadBinMsg := '';
      Problem := 0;
      IF abi.OneSetting THEN
        begin
          IF NOT abi.GlobalInfo.HasBin THEN Problem := 1
          ELSE IF abi.GlobalInfo.BinPercent = 0 THEN Problem := 1;
          IF (Problem = 1) THEN BadBinMsg := 'RECYCLE BIN SETTINGS '+
          'ERROR:'#13#10#13#10'The Recycle Bin is configured to use'+
          ' one setting for all drives. It is either disabled, or i'+
          't is set to use zero percent of each drive.'#13#10#13#10+
          '%s'#13#10#13#10'Please see the help for instructions on '+
          'correcting this problem';
        end
      ELSE
        begin
          S := '';
          FOR N := 0 TO 25 DO
            WITH abi.DrivesInfo[N] DO
              begin
                IF NOT InReg THEN Continue;
                IF NOT Local THEN Continue;
                IF NOT HasBin THEN
                  begin
                    Problem := 2;
                    S := S + Char(Ord('a')+N);
                  end
                ELSE IF BinPercent = 0 THEN
                  begin
                    Problem := 2;
                    S := S + Char(Ord('a')+N);
                  end;
              end;
          IF (Problem = 2) THEN
            begin
              IF Length(S) = 1 THEN
                S := 'drive '+S+':\'
              ELSE
                begin
                  L := Length(S);
                  Insert(':\ and ', S, Length(S));
                  FOR N := L-2 DOWNTO 1 DO
                    Insert(':\, ', S, N+1);
                  S := 'drives '+S+':\';
                end;
              BadBinMsg := 'RECYCLE BIN SETTINGS ERROR:'#13#10#13+
              #10'The Recycle Bin is configured to use independent '+
              'settings for different drives. On '+S+', the Recycle'+
              ' Bin is either disabled, or it is set to use zero pe'+
              'rcent of the drive.'#13#10#13#10'%s'#13#10#13#10'Ple'+
              'ase see the help for instructions on correcting this'+
              ' problem';
            end;
        end;
    end;
  Result := Problem > 0;
end;

end.


